﻿using DotNetCore.CAP;
using Hotline.Application.Quality;
using Hotline.Orders;
using Hotline.Orders.Notifications;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Quality;
using Hotline.Users;
using Mapster;
using MediatR;
using XF.Domain.Repository;

namespace Hotline.Application.Orders.OrderVisitHandler;
public class OrderVisitUpdateHandler : INotificationHandler<UpdateOrderVisitNotify>
{
    private readonly IOrderRepository _orderRepository;
    private readonly IRepository<OrderVisitDetail> _orderVisitDetailRepository;

    public OrderVisitUpdateHandler(IOrderRepository orderRepository, IRepository<OrderVisitDetail> orderVisitDetailRepository)
    {
        _orderRepository = orderRepository;
        _orderVisitDetailRepository = orderVisitDetailRepository;
    }

    public async Task Handle(UpdateOrderVisitNotify notification, CancellationToken cancellationToken)
    {
        var visit = notification.orderVisit;

        var orgDetail = await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == visit.Id && m.VisitTarget == EVisitTarget.Org).FirstAsync(cancellationToken);
        var seatDetail = await _orderVisitDetailRepository.Queryable()
            .Where(m => m.VisitId == visit.Id && m.VisitTarget == EVisitTarget.Seat).FirstAsync(cancellationToken);

        await _orderRepository.Updateable()
            .SetColumns(m => m.OrgProcessingResults == orgDetail.OrgProcessingResults)
            .SetColumns(m => m.SeatEvaluate == seatDetail.SeatEvaluate)
            .Where(m => m.Id == visit.OrderId)
            .ExecuteCommandAsync(cancellationToken);
    }
}
