﻿using System.ComponentModel;
using Hotline.Share.Dtos.File;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Requests;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order
{
    public record QueryOrderDto : PagedKeywordRequest
    {
        /// <summary>
        /// 省本地编号（√）
        /// </summary>
        public string ProvinceNo { get; set; }

        /// <summary>
        /// 本地编号（√）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 诉求内容
        /// </summary>
        //public string? Content { get; set; }

        /// <summary>
        /// 受理类型（√）
        /// </summary>
        public string? AcceptType { get; set; }
        //public List<string> AcceptTypes { get; set; } = new();

        /// <summary>
        /// 来源渠道（√）
        /// </summary>
        public string? Channel { get; set; }
        //public List<string> Channels { get; set; } = new();

        /// <summary>
        /// 转接号码（转接来源）（√）
        /// </summary>
        public string? TransferPhone { get; set; }

        ///// <summary>
        ///// 热点分类（√）
        ///// </summary>
        //public List<string> HotspotIds { get; set; } = new();

        /// <summary>
        /// 热点分类关键词
        /// </summary>
        public string? Hotspot { get; set; }

        /// <summary>
        /// 接办部门（√）
        /// </summary>
        public string? OrgId { get; set; }
        //public List<string> OrgCodes { get; set; } = new();

        public string? OrgLevelOneName { get; set; }
        /// <summary>
        /// 接办名称（综合查询使用）
        /// </summary>
        public string? ActualHandleOrgName { get; set; }

        /// <summary>
        /// 受理坐席名字或工号（×）
        /// </summary>
        public string? NameOrNo { get; set; }

        /// <summary>
        /// 受理时间(工单创建时间)（√）
        /// </summary>
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }

        /// <summary>
        /// 来电号码（×）
        /// </summary>
        public string? FromPhone { get; set; }


        /// <summary>
        /// 来电或联系电话（√）
        /// </summary>
        public string? PhoneNo { get; set; }

        /// <summary>
        /// 推送分类（√）(单选)
        /// </summary>
        public string? PushTypeCode { get; set; }

        /// <summary>
        /// 期满时间（√）
        /// </summary>
        public DateTime? ExpiredTimeStart { get; set; }
        public DateTime? ExpiredTimeEnd { get; set; }


        /// <summary>
        /// 工单状态（√）
        /// </summary>
        public EOrderStatus? Status { get; set; }
        //public List<EOrderStatus> Statuses { get; set; } = new();

        /// <summary>
        /// 接办人（√）
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 是否甄别（√）
        /// </summary>
        public bool? IsScreen { get; set; }

        /// <summary>
        /// 当前办理节点（√）（单选）
        /// </summary>
        public string? CurrentStepCode { get; set; }
        /// <summary>
        /// 办结时间（√）
        /// </summary>
        public DateTime? ActualHandleTimeStart { get; set; }
        public DateTime? ActualHandleTimeEnd { get; set; }

        /// <summary>
        /// 是否超期（√）
        /// </summary>
        public bool? IsOverTime { get; set; }

        /// <summary>
        /// 来电主体（√）(单选)
        /// </summary>
        public EIdentityType? IdentityType { get; set; }

        /// <summary>
        /// 来电人姓名（×）
        /// </summary>
        public string? FromName { get; set; }

        /// <summary>
        /// 区域（×）(多选)
        /// </summary>
        public string? AreaCode { get; set; }
        //public List<string> AreaCodes { get; set; } = new();

        /// <summary>
        /// 是否省工单（空为全部  true为省工单 false为市工单）
        /// </summary>
        public bool? IsProvinceOrder { get; set; }


        /// <summary>
        /// 是否敏感词
        /// </summary>
        public bool? IsSensitiveWord { get; set; }

        /// <summary>
        /// 敏感词
        /// </summary>
        public string? SensitiveWord { get; set; }

        /// <summary>
        /// 是否紧急
        /// </summary>
        public bool? IsUrgent { get; set; }

        /// <summary>
        /// 省来源分类 1：政民互动直派，2：政民互动，3：省12345
        /// </summary>
        public string? ProvinceChannel { get; set; }

        /// <summary>
        /// 内容检索
        /// </summary>
        public string? ContentRetrieval { get; set; }

        /// <summary>
        /// 归档方式
        /// </summary>
        public FiledType? FiledType { get; set; }
        /// <summary>
        /// 省交办编号
        /// </summary>
        public string? ReceiveProvinceNo { get; set; }

        /// <summary>
        /// 受理情况 true :已签收  false:未签收  空：全部
        /// </summary>
        public bool? IsSgin { get; set; }

        /// <summary>
        /// 工单标签Code
        /// </summary>
        public string? OrderTagCode { get; set; }

        /// <summary>
        /// 排序字段
        /// </summary>
        public string? SortField { get; set; }

        /// <summary>
        /// 排序方式  0 升序 1 降序
        /// </summary>
        public int? SortRule { get; set; }

        /// <summary>
        /// 重点标记
        /// </summary>
        public string? FocusOnEvents { get; set; }

        /// <summary>
        /// 是否保密
        /// </summary>
        public bool? IsSecret { get; set; }

    }


	public enum FiledType
    {
        [Description("中心归档")]
        CenterFiled = 10,

        [Description("部门归档")]
        OrgFiled = 20,
    }

    public record QueryOrderRedoRecordDto : PagedKeywordRequest
    {

    }


    public record CanDelayOrderListDto : PagedKeywordRequest
    {

    }

    public record CancelDelayDto
    {
        public string Id { get; set; }
    }

    public record ApplyDelayDto
    {
        public string OrderId { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }

        public string DelayReason { get; set; }

        public List<FileDto> Files { get; set; }
    }

    public record OrderDelayWebDto
    {

    }

    public class PublishOrderDelayDto
    {
        public string Id { get; set; }

        /// <summary>
        /// 工单编号（冗余）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        public OrderDto? Order { get; set; }

        /// <summary>
        /// 延期申请时间
        /// </summary>
        public DateTime ApplyDelayTime { get; set; }

        /// <summary>
        /// 申请部门Code
        /// </summary>
        public string ApplyOrgCode { get; set; }

        /// <summary>
        /// 申请部门名称
        /// </summary>
        public string ApplyOrgName { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string EmployeeId { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public string? EmployeeName { get; set; }

        /// <summary>
        /// 延期申请时限
        /// </summary>
        public int DelayNum { get; set; }

        /// <summary>
        /// 延期申请单位
        /// </summary>
        public ETimeType DelayUnit { get; set; }

        /// <summary>
        /// 延期申请理由
        /// </summary>
        public string DelayReason { get; set; }

        /// <summary>
        /// 申请前期满时间
        /// </summary>
        public DateTime? BeforeDelay { get; set; }

        /// <summary>
        /// 申请后期满时间
        /// </summary>
        public DateTime? AfterDelay { get; set; }

        /// <summary>
        /// 审批状态
        /// </summary>
        public EDelayState DelayState { get; set; }

        /// <summary>
        /// 是否省延期
        /// </summary>
        public bool IsProDelay { get; set; }

        public List<FileJson> FileJson { get; set; }
    }

    public record OrderDelayDto
    {
        /// <summary>
        /// 工单编号（冗余）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        public OrderDto? Order { get; set; }

        /// <summary>
        /// 延期申请时间
        /// </summary>
        public DateTime ApplyDelayTime { get; set; }

        /// <summary>
        /// 申请部门Code
        /// </summary>
        public string ApplyOrgCode { get; set; }

        /// <summary>
        /// 申请部门名称
        /// </summary>
        public string ApplyOrgName { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string EmployeeId { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public string? EmployeeName { get; set; }

        /// <summary>
        /// 延期申请时限
        /// </summary>
        public int DelayNum { get; set; }

        /// <summary>
        /// 延期申请单位
        /// </summary>
        public ETimeType DelayUnit { get; set; }

        public string DelayUnitText => DelayUnit.GetDescription();

        /// <summary>
        /// 延期申请理由
        /// </summary>
        public string DelayReason { get; set; }

        /// <summary>
        /// 申请前期满时间
        /// </summary>
        public DateTime? BeforeDelay { get; set; }

        /// <summary>
        /// 申请后期满时间
        /// </summary>
        public DateTime? AfterDelay { get; set; }

        /// <summary>
        /// 审批状态
        /// </summary>
        public EDelayState DelayState { get; set; }

        public string DelayStateText => DelayState.GetDescription();

        /// <summary>
        /// 是否省延期
        /// </summary>
        public bool IsProDelay { get; set; }

        /// <summary>
        /// 流程ID
        /// </summary>
        public string? WorkflowId { get; set; }

        public DateTime CreationTime { get; set; }

        public string Id { get; set; }

        public string? CreatorId { get; set; }

        public string? CreatorName { get; set; }

        public string? CreatorOrgId { get; set; }

        public string? CreatorOrgName { get; set; }

        public int CreatorOrgLevel { get; set; }

        /// <summary>
        /// 一级部门Id
        /// </summary>
        public string? AreaId { get; set; }

        /// <summary>
        /// 当前办理节点
        /// </summary>
        public string? CurrentStepName { get; set; }
        /// <summary>
        /// 当前办理人
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 是否可办理
        /// </summary>
        public bool IsCanHandle { get; set; }

        public List<FileDto> Files { get; set; }

        public List<FileJson> FileJson { get; set; }

        /// <summary>
        /// 办理 true  审批 false 
        /// </summary>
        public bool Handle { get; set; }

        /// <summary>
        /// 自动延期次数
        /// </summary>
        public int? AutomaticDelayNum { get; set; }
	}


    public record DelayCalcEndTimeDto
    {
        public DateTime? BeginTime { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }
    }

    public record DelayListDto : PagedKeywordRequest
    {
        /// <summary>
        /// 是否已办 true:已办 false:待办
        /// </summary>
        public bool? IsApply { get; set; }

        public EDelayState? DelayState { get; set; }

        /// <summary>
        /// 延期状态快捷查询
        /// </summary>

        public EQueryDelayState? QueryDelayState { get; set; }

		/// <summary>
		/// 0 全部  1 我的
		/// </summary>
		public int? DataScope { get; set; }


		/// <summary>
		/// 当前办理节点
		/// </summary>
		public string? CurrentStepName { get; set; }
		/// <summary>
		/// 当前办理人
		/// </summary>
		public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 一级部门
        /// </summary>
		public string? OrgLevelOneName { get; set; }

		/// <summary>
		/// 延期申请时间
		/// </summary>
		public DateTime? StartCreationTime { get; set; }

		/// <summary>
		/// 延期申请时间
		/// </summary>
		public DateTime? EndCreationTime { get; set; }
	}

    public record ApplySuperviseDto
    {
        public string OrderId { get; set; }
    }

    public record SuperviseListDto : PagedKeywordRequest
    {
        /// <summary>
        /// 督办回复状态
        /// </summary>
        public int? SuperviseState { get; set; }

        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }
    }

    public record ScreenListDto : PagedKeywordRequest
    {

		/// <summary>
		/// 0 全部  1 我的
		/// </summary>
		public int? DataScope { get; set; }

        /// <summary>
        /// 甄别类型查询条件
        /// </summary>
        public EQueryOrderScreenType? QueryScreenType { get; set; }

		/// <summary>
		/// 甄别申请状态选项卡
		/// </summary>
		public EScreenStatus? TabStatus { get; set; }

		/// <summary>
		/// 甄别申请状态
		/// </summary>
		public EScreenStatus? Status { get; set; }

		/// <summary>
		/// 编码
		/// </summary>
		public string? No { get; set; }

		/// <summary>
		/// 标题
		/// </summary>
		public string? Title { get; set; }

		/// <summary>
		/// 受理类型
		/// </summary>
		public string? AcceptType { get; set; }

		/// <summary>
		/// 热点类型
		/// </summary>
		public string? HotspotSpliceName { get; set; }

		/// <summary>
		/// 来源渠道（电话、网站、APP等）
		/// </summary>
		public string? SourceChannel { get; set; }

		/// <summary>
		/// 回访部门名称
		/// </summary>
		public string? VisitOrgName { get; set; }

		/// <summary>
		/// 甄别申请部门
		/// </summary>
		public string? CreatorOrgName { get; set; }

		/// <summary>
		/// 是否省工单
		/// </summary>
		public bool? IsProvince { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string? CreatorName { get; set; }

        /// <summary>
        /// 甄别申请时间
        /// </summary>
		public DateTime? CreationTimeStart { get; set; }

        /// <summary>
        /// 甄别申请时间
        /// </summary>
		public DateTime? CreationTimeEnd { get; set; }

        public string? OrderId { get; set; }

        /// <summary>
        /// 甄别查询来源  1 甄别待办  2 甄别列表
        /// </summary>
        public int source { get; set; }

        /// <summary>
        /// 重提甄别
        /// </summary>
        public bool? IsSendBackApplyNum { get; set; }

		//甄别列表
		/// <summary>
		/// 一级部门名称
		/// </summary>
		public string? OrgLevelOneName { get; set; }

		/// <summary>
		/// 接办部门
		/// </summary>
		public string? CurrentHandleOrgName { get; set; }

		/// <summary>
		/// 受理时间
		/// </summary>
		public DateTime? CreationTime { get; set; }

		/// <summary>
		/// 受理时间
		/// </summary>
		public DateTime? EndCreationTime { get; set; }

		/// <summary>
		/// 接办时间
		/// </summary>
		public DateTime? CurrentHandleTime { get; set; }

		/// <summary>
		/// 接办时间
		/// </summary>
		public DateTime? EndCurrentHandleTime { get; set; }

		/// <summary>
		/// 归档时间（暂为流程结束时间，因流程结束自动归档）
		/// </summary>
		public DateTime? FiledTime { get; set; }

		/// <summary>
		/// 归档时间（暂为流程结束时间，因流程结束自动归档）
		/// </summary>
		public DateTime? EndFiledTime { get; set; }

		/// <summary>
		/// 回访时间
		/// </summary>
		public DateTime? VisitTime { get; set; }

		/// <summary>
		/// 回访时间
		/// </summary>
		public DateTime? EndVisitTime { get; set; }

		/// <summary>
		/// 联系电话
		/// </summary>
		public string? Contact { get; set; }

		/// <summary>
		/// 来电电话
		/// </summary>
		public string? FromPhone { get; set; }
	}

    public record MayScreenListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 甄别退回  0  全部  1 是  2 否
	    /// </summary>
	    public int? ScreenSendBack { get; set; }

	    /// <summary>
	    /// 工单编号
	    /// </summary>
	    public string? No { get; set; }

	    /// <summary>
	    /// 标题
	    /// </summary>
	    public string? Title { get; set; }

	    /// <summary>
	    /// 是否省工单
	    /// </summary>
	    public bool? IsProvince { get; set; }

	    /// <summary>
	    /// 受理类型
	    /// </summary>
	    public string? AcceptType { get; set; }

	    /// <summary>
	    /// 热点类型
	    /// </summary>
	    public string? HotspotSpliceName { get; set; }

	    /// <summary>
	    /// 来源渠道（电话、网站、APP等）
	    /// </summary>
	    public string? SourceChannel { get; set; }

	    /// <summary>
	    /// 一级部门名称
	    /// </summary>
	    public string? OrgLevelOneName { get; set; }

	    /// <summary>
	    /// 接办部门
	    /// </summary>
	    public string? CurrentHandleOrgName { get; set; }

	    /// <summary>
	    /// 回访部门名称
	    /// </summary>
	    public string? VisitOrgName { get; set; }

        /// <summary>
        /// 受理时间
        /// </summary>
        public DateTime? CreationTime { get; set; }

        /// <summary>
        /// 受理时间
        /// </summary>
        public DateTime? EndCreationTime { get; set; }

		/// <summary>
		/// 接办时间
		/// </summary>
		public DateTime? CurrentHandleTime { get; set; }

		/// <summary>
		/// 接办时间
		/// </summary>
		public DateTime? EndCurrentHandleTime { get; set; }

		/// <summary>
		/// 归档时间（暂为流程结束时间，因流程结束自动归档）
		/// </summary>
		public DateTime? FiledTime { get; set; }

		/// <summary>
		/// 归档时间（暂为流程结束时间，因流程结束自动归档）
		/// </summary>
		public DateTime? EndFiledTime { get; set; }

		/// <summary>
		/// 回访时间
		/// </summary>
		public DateTime? VisitTime { get; set; }

		/// <summary>
		/// 回访时间
		/// </summary>
		public DateTime? EndVisitTime { get; set; }


		public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }
        public bool? IsHomePage { get; set; }
        
        /// <summary>
        /// 会签类型
        /// </summary>
        public ECounterSignType? CounterSignType { get; set; }

        /// <summary>
        /// 部门办件结果
        /// </summary>
        public string? OrgProcessingResults { get; set; }

        /// <summary>
        /// 部门办件态度
        /// </summary>
        public string? OrgHandledAttitude { get; set; }

        /// <summary>
        /// 不满意原因
        /// </summary>
        public string? OrgNoSatisfiedReason { get; set; }

		/// <summary>
		/// 甄别类型
		/// </summary>
		public EOrderScreenType? ScreenType { get; set; }

		/// <summary>
		/// 受理坐席名字或工号（×）
		/// </summary>
		public string? NameOrNo { get; set; }
	}

    public record UrgeListDto : PagedKeywordRequest
    {
        /// <summary>
        /// 回复状态
        /// </summary>
        public int? UrgeState { get; set; }
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }
    }

    public class QueryRepeatableEventDto
    {
        public string? HotspotSpliceName { get; set; }

        public string? Address { get; set; }
    }


    public enum EPublicState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 公开
        /// </summary>
        [Description("公开")]
        Pub = 1,
        /// <summary>
        /// 不公开
        /// </summary>
        [Description("不公开")]
        NoPub = 2,
    }


    public enum EPubState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 已发布
        /// </summary>
        [Description("已发布")]
        Pub = 1,
        /// <summary>
        /// 未发布
        /// </summary>
        [Description("未发布")]
        NoPub = 2
    }

    public enum EQueryDelayState
    {

	    /// <summary>
	    /// 全部
	    /// </summary>
	    [Description("全部")]
	    All = -1,

		/// <summary>
		/// 待审批
		/// </summary>
		[Description("待审批")]
	    Examining = 0,

	    /// <summary>
	    /// 审批通过
	    /// </summary>
	    [Description("审批通过")]
	    Pass = 1,

		/// <summary>
		/// 审批拒绝
		/// </summary>
		[Description("审批拒绝")]
	    NoPass = 2,
    }
	public enum EQueryOrderScreenType
	{
		/// <summary>
		/// 全部
		/// </summary>
		[Description("全部")]
		All = -1,

		/// <summary>
		/// 部门办件
		/// </summary>
		[Description("部门办件")]
		Org = 0,

		/// <summary>
		/// 中心话务
		/// </summary>
		[Description("中心话务")]
		Seat = 1,
	}

}
